%
% Animacia systemu dvoch zasobnikov kvapaliny
% Tatiana Lipnicanova
% marec 2004
%
function [sys, x0, str, ts] = tanks(t, x, u, flag)
global Stime q0s q0 k11 k22 F1 F2 intr h1s h2s
global h1s_q0 h2s_q0 h1s_q0s h2s_q0s my
global level_last stream_old koef data_last

switch flag,
case 0,
    [sys, x0, str,ts] = mdlInitializeSizes(intr);
case 2,
    sys = mdlUpdate(t, x, u, h1s, h2s, intr);
case 3,
    mdlOutput;
case 9,
    mdlTerminate;
case { 1, 4 },
sys = [];
otherwise
error(['Unexpexted flag: ', num2str(flag)]);
end

% --------
function [sys, x0, str, ts] = mdlInitializeSizes(intr)
global q0s h1s h2s h1s_q0s h2s_q0s my
global level_last stream_old koef data_last

sizes = simsizes;
sizes.NumContStates = 0;
sizes.NumDiscStates = 0;
sizes.NumOutputs = 0;
sizes.NumInputs = 3;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;
sys = simsizes(sizes);
x0 = [];
str = [];
ts = [0.5 0];

level_last = zeros(1,2);
data_last = [];
stream_old = zeros(1,3);
koef = [];
h1s_q0s = h1s;
h2s_q0s = h2s;

% Load background graphics
bg = background;

for i = 1:2
    %  set possition of displaying of the actual hight of water level    
    set(findobj('Tag', my.tag{1,i}),'Position', [bg.X_t2(i) bg.Y_t(i) 0]);
    set(findobj('Tag', my.tag{2,i}),'Position', [bg.X_t1(i) bg.Y_t(i) 0]);
end
for i = 1:3
    %  displaying of the actual water flow
    set(findobj('Tag', my.tag{3,i}),'Position', [bg.X_t3(i) bg.Y_t3(i) 0]);
    set(findobj('Tag', my.tag{4,i}),'Position', [bg.X_t4(i) bg.Y_t3(i) 0]);
end
set(findobj('Tag', my.tag{3,1}),'String', sprintf('%.2f',q0s));
% --------

% --------
function sys = mdlUpdate(t, x, u, h1s, h2s, intr)
global k11 k22 F1 F2 h1s_q0 h2s_q0 h1s_q0s h2s_q0s q0s q0 Stime my
global level_last data_last stream_old koef

axes(findobj('Tag','picture'));

% draw the water
[level_last, stream_old koef] = create_frame(t, u(1:2), h1s, h2s, intr, level_last, stream_old, koef, k11, k22);
draw_background('continue');

% graphs
data_last = graph(t, u(1:2), data_last, h1s, h2s);

% a step between two definable water flows at a specified time
stepwf(t, Stime, q0s, q0, k11, k22, intr);

sys = [];
% --------

% --------
function mdlOutput
% Nothing happens here
% --------

% --------
function mdlTerminate
% Nothing happens here
% --------

% --------
function [backLL, backSO, backK] = create_frame(t, level, h1s, h2s, intr, level_last, stream_old, koef, k11, k22)
global bg h1s_q0 h2s_q0 h1s_q0s h2s_q0s q0s q0 Stime my

color_water = [0.62 0.753 0.922];
color_water2 = [0.55 0.66 0.93];
axes(findobj('Tag','picture'));

tank_max = bg.left_side(2,2)-bg.left_side(1,2);
level_max = 1.1*(max([h1s_q0s h2s_q0s h1s_q0 h2s_q0]));

if level_max ~= 0
    koef = tank_max/level_max;
end
backK = koef;

tank_posit = [bg.bottom_part11(1,2) bg.bottom_part1(1,2)];    % y-axes of left down point of the tank
% determine the water height in the tanks
Y1 = tank_posit(1) + koef*level(1);
Y2 = tank_posit(2) + koef*level(2);
X = [bg.bottom_part11(1,1) bg.bottom_part21(11,1) bg.bottom_part1(1,1) bg.bottom_part2(11,1)];

% display the actual water flow  if the value was changed
switch intr
    case 0
        q1 = k11*sqrt(level(1));
    case 1,
        q1 = k11*sqrt(level(1)-level(2));
end
q2 = k22*sqrt(level(2));
set(findobj('Tag',my.tag{3,2}),'string', sprintf('%.2f', q1));
set(findobj('Tag',my.tag{3,3}),'string',sprintf('%.2f', q2));            

flow_max = max(q0, q0s);
if q0 ~= q0s & t >= Stime
    percF0 = q0/flow_max;
else
    percF0 = q0s/flow_max;
end
percF1 = q1/flow_max;
percF2 = q2/flow_max;

% draw water entering the left tank
if q0 == 0 & t >= Stime
    set(stream_old(1),'Visible','off')
    streamL = 0;
else
    if level(1) ~= 0
        if (percF0 > 0 & percF0 <= 0.3)
            X_stream = [bg.pipeline_left2(12,1)*ones(1,2)+2*ones(1,2) bg.pipeline_left2(12,1)*ones(1,2)+9*ones(1,2)];
        elseif (percF0 > 0.3 & percF0 <= 0.7)
            X_stream = [bg.pipeline_left2(12,1)*ones(1,2)+2*ones(1,2) bg.pipeline_left2(12,1)*ones(1,2)+16*ones(1,2)];
        else
            X_stream = [bg.pipeline_left2(12,1)*ones(1,2)+2*ones(1,2) bg.pipeline_left2(12,1)*ones(1,2)+23*ones(1,2)];
        end
        Y_stream = [Y1 bg.roof_part31(4,2) bg.roof_part31(4,2) Y1];
        
        if stream_old(1) ~= 0
            delete(stream_old(1))
        end
        streamL = fill(X_stream, Y_stream, color_water, 'EdgeColor', 'none','EraseMode','background', 'Tag', 'StreamL');
    else
        streamL = 0;
    end
end

% draw water entering the right tank
if level(1) ~= 0
    if intr == 0
        if (percF1 > 0 & percF1 <= 0.3)
            X_stream = [bg.pipeline_m2(7,1)*ones(1,2)+2*ones(1,2) bg.pipeline_m2(7,1)*ones(1,2)+9*ones(1,2)];
        elseif (percF1 > 0.3 & percF1 <= 0.7)
            X_stream = [bg.pipeline_m2(7,1)*ones(1,2)+2*ones(1,2) bg.pipeline_m2(7,1)*ones(1,2)+16*ones(1,2)];
        else
            X_stream = [bg.pipeline_m2(7,1)*ones(1,2)+2*ones(1,2) bg.pipeline_m2(7,1)*ones(1,2)+23*ones(1,2)];
        end
        Y_stream = [Y2 bg.roof_part3(4,2) bg.roof_part3(4,2) Y2];
        
        if stream_old(2) ~= 0
            delete(stream_old(2))
        end
        streamM = fill(X_stream, Y_stream, color_water, 'EdgeColor', 'none','EraseMode','background');
    else
        streamM = 0;
    end
else
    set(stream_old(2),'Visible','off')
    streamM = 0;
end

% draw water streaming from the right tank
if level(2) ~= 0
    if (percF2 > 0 & percF2 <= 0.3)
        X_stream = [bg.pipeline_right2(5,1)*ones(1,2)+2*ones(1,2) bg.pipeline_right2(5,1)*ones(1,2)+9*ones(1,2)];
    elseif (percF2 > 0.3 & percF2 <= 0.7)
        X_stream = [bg.pipeline_right2(5,1)*ones(1,2)+2*ones(1,2) bg.pipeline_right2(5,1)*ones(1,2)+16*ones(1,2)];
    else
        X_stream = [bg.pipeline_right2(5,1)*ones(1,2)+2*ones(1,2) bg.pipeline_right2(5,1)*ones(1,2)+23*ones(1,2)];
    end
    Y_stream = [bg.bottom_tank(3) bg.pipeline_right2(7,2) bg.pipeline_right2(7,2) bg.bottom_tank(3)];
    
    if stream_old(3) ~= 0
        delete(stream_old(3))
    end
    streamR = fill(X_stream, Y_stream, color_water, 'EdgeColor', 'none','EraseMode','background');
else
    set(stream_old(3),'Visible','off')
    streamR = 0;
end

backSO = [streamL streamM streamR];
% ------

% draw water in the left tank
if level_last(1) ~= level(1)
    % delete old handles
    ob_wl2 = findobj('Tag', 'wlev1');
    if size(ob_wl2,1)>11 & size(ob_wl2,1)< 23
        delete(ob_wl2(1:11))
    elseif size(ob_wl2,1) > 23
        delete(ob_wl2(1:11),ob_wl2(23:33))
    end
    ob_w11 = findobj('Tag','wtank1');
    if size(ob_w11,1) > 0
        delete(ob_w11)
    end
    
    % draw the first part of the water in left tank
    fill([X(1) X(1) X(2) X(2) X(1)],[Y1 tank_posit(1) tank_posit(1) Y1 Y1], ...
        color_water, 'EdgeColor', 'none','Tag','wtank1','EraseMode','background');
    
    Z = [0 koef*level(1)];
    for i = 1 : 2
        % second part of the water
        fill([bg.bottom_part11(11,1) bg.bottom_part21(1,1) bg.bottom_part21(11,1) bg.bottom_part41(1,1)  bg.bottom_part31(11,1) bg.bottom_part31(1,1)], ...
            [Z(i)*ones(11,1)+bg.bottom_part11(11,2) Z(i)*ones(11,1)+bg.bottom_part21(1,2) Z(i)*ones(11,1)+bg.bottom_part21(11,2) ...
                Z(i)*ones(11,1)+bg.bottom_part41(1,2)  Z(i)*ones(11,1)+bg.bottom_part31(11,2) Z(i)*ones(11,1)+bg.bottom_part31(1,2)], ...
            color_water2,'LineWidth',1.2,'Tag', 'wlev1','EraseMode','background');
    end
    if level(1) == 0
        delete(findobj('Tag', 'wlev1'))
    end
end

%draw water in the right tank
if level_last(2) ~= level(2)
    % delete old handles
    ob_w22 = findobj('Tag', 'wlev2');
    if size(ob_w22,1)>11 & size(ob_w22,1)< 23
        delete(ob_w22(1:11))
    elseif size(ob_w22,1) > 23
        delete(ob_w22(1:11),ob_w22(23:33))
    end
    
    ob_w21 = findobj('Tag', 'wtank2');
    if size(ob_w21,1) > 0
        delete(ob_w21)
    end
    % draw the first part of water in the right tank
    fill([X(3) X(3) X(4) X(4) X(3)],[Y2 tank_posit(2) tank_posit(2) Y2 Y2], ...
        color_water, 'EdgeColor', 'none','Tag', 'wtank2','EraseMode','background');
    
    Z = [0 koef*level(2)];
    for i = 1 : 2
        % second part of water
        fill([bg.bottom_part1(11,1) bg.bottom_part2(1,1) bg.bottom_part2(11,1) bg.bottom_part4(1,1)  bg.bottom_part3(11,1) bg.bottom_part3(1,1)], ...
            [Z(i)*ones(11,1)+bg.bottom_part1(11,2) Z(i)*ones(11,1)+bg.bottom_part2(1,2) Z(i)*ones(11,1)+bg.bottom_part2(11,2) ...
                Z(i)*ones(11,1)+bg.bottom_part4(1,2)  Z(i)*ones(11,1)+bg.bottom_part3(11,2) Z(i)*ones(11,1)+bg.bottom_part3(1,2)], ...
            color_water2,'Tag', 'wlev2','EraseMode','background');
    end
    if level(2) == 0
        delete(findobj('Tag', 'wlev2'))
    end
end


% display the actual water level if the value was changed
for i = 1:2
    if level_last(i) ~= level(i)
        set(findobj('Tag', my.tag{2,i}),'string',sprintf('%.2f',level(i)));
        level_last(i) = level(i);
    end
end
backLL = level_last;

if ~isempty(findobj('String','h1(t)='))
    if abs(level(1)-h1s_q0) < 0.006
        set(findobj('Tag',my.tag{1,1},'String','h1(t)='),'String','h1s =')
    end   
elseif ~isempty(findobj('String','h2(t)='))
    if abs(level(2)-h2s_q0) < 0.006
        set(findobj('Tag',my.tag{1,2},'String','h2(t)='),'String','h2s =')
    end
end
if abs(q0-q1) < 0.006
    set(findobj('Tag',my.tag{4,2}),'String', 'q1s =');
elseif abs(q0-q2) < 0.006
    set(findobj('Tag',my.tag{4,3}),'String', 'q2s =');
end
% --------

% --------
function back = graph(t, level, data_last, h1s, h2s)
global  h1s_q0 h2s_q0

if t == 0
    level_max = max([h1s h2s h1s_q0 h2s_q0]);
    if level_max == 0
        Y_graf = 3;
    else
        Y_graf = ceil(level_max);
    end
set(findobj('Tag','graphL'),'YLim',[0 Y_graf])
end

if t > 5
    set(findobj('Tag','graphL'),'XLim',[0 t])
end

if size(data_last) == [0 0]
    x_old = [];
    y1_old = [];
    y2_old = [];
else
    x_old = data_last(:,1);
    y1_old = data_last(:,2);
    y2_old = data_last(:,3);
end
x_data = [x_old; t];
y1_data = [y1_old; level(1)];
y2_data = [y2_old; level(2)];

if size(x_data,1) > 1
    axes(findobj('Tag','graphL'))
    hold on
    plot(x_data, y1_data, 'Color', [0 0.854 0]);
    plot(x_data, y2_data, 'Color', [0.85 0 0]);
    legend('h1(t)','h2(t)',0)
    hold off
end

back = [x_data y1_data y2_data];
% --------

% --------
function stepwf(t, Stime, q0s, q0, k11, k22, intr)
global F1 F2 h1s_q0 h2s_q0 h1s h2s my

if (t == Stime) & (q0 ~= q0s)
    set(findobj('Tag','h1sresult'),'String',sprintf('%.2f',h1s_q0))
    set(findobj('Tag','h2sresult'),'String',sprintf('%.2f',h2s_q0))
    set(findobj('Tag',my.tag{3,1}),'String', sprintf('%.2f',q0))
    set(findobj('Tag',my.tag{1,1}),'String','h1(t)=')
    set(findobj('Tag',my.tag{1,2}),'String','h2(t)=')
    h1s = h1s_q0;
    h2s = h2s_q0;
end
% --------
