%
% Animacia systemu dvoch zasobnikov kvapalinu (s/bez interakcie)
% Tatiana Lipnicanova
% april 2004
%
function [sys, x0, str, ts] = tanksnew(t, x, u, flag, kk, ff, hh, intr, zas, anim)
global bg my

switch flag,
    case 0,
        [sys, x0, str, ts] = mdlInitializeSizes(intr, zas, anim);
    case 2,
        sys = mdlUpdate(u, kk, intr, zas);
    case 3,
        mdlOutput;
    case 9,
        mdlTerminate;
    case { 1, 4 },
        sys = [];
    otherwise
        error(['Unexpexted flag: ', num2str(flag)]);
end

function [sys, x0, str, ts] = mdlInitializeSizes(intr, zas, anim)
global my bg level_last data_last koef onetime tank_max

sizes = simsizes;
sizes.NumContStates = 0;
sizes.NumDiscStates = 0;
sizes.NumOutputs = 0;
sizes.NumInputs = 3;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;
sys = simsizes(sizes);
x0 = [];
str = [];
ts = [0.5 0];

level_last = zeros(1,2);
data_last = [];
onetime = zeros(1,7);

% Load background graphics
bg = background(intr);

tank_max = bg.left_side(2,2)-bg.left_side(1,2);
koef = tank_max/zas(1);

if ~isempty(findobj('Name', 'Two tanks system'))
    close('Two tanks system');
end
if anim ~= 0 
    % Create animation window
    my.objHandl(1) = figure('Color',[0.75 0.75 0.75], ...
        'MenuBar','none', ...
        'Name','Two tanks system', ...
        'NumberTitle', 'off', ...
        'Position',[200 200 400 450], ...
        'PaperUnits','points', ...
        'Resize', 'off', ...
        'DoubleBuffer', 'on', ...
        'ToolBar','none');
    
    % animation axis
    my.objHandl(2) = axes('Parent',my.objHandl(1),...
        'Box','on', ...
        'Color',[1 1 1], ...
        'Layer', 'top', ...
        'NextPlot','add', ...
        'Position',[0.05 0.05 0.90 0.90], ...
        'Tag','picture', ...
        'TickDir','out', ...
        'Visible','on', ...
        'XColor',[0.75 0.75 0.75],...
        'XLim',[-50 1550], ...
        'YColor',[0.75 0.75 0.75], ...
        'YLim',[-50 1200]);
    
    draw_background('begin',intr);
    
    handle_t1 = zeros(1,2);
    handle_t2 = zeros(1,3);
    
    my.tag = {'h1s =' 'h2s =' ''; 'nh1' 'nh2' ''; 'nq0' 'nq1' 'nq2'; 'q0s =' 'q1(t) =' 'q2(t) ='; 'perLT' 'perRT' ''};
    
    for i = 1 : 2,
        % object which display the  actual liquid level
        % 1.25
        handle_t1(i) = text('Parent', my.objHandl(2), ...
            'Position', [bg.X_t1(i) bg.Y_t(i) 0], ...
            'Color', [0 0.3 0.8], ...
            'EraseMode', ' background', ...
            'FontAngle', 'italic', ...
            'FontSize', 8.5, ...
            'HorizontalAlignment','left', ...
            'String', '0.00', ...
            'Tag', my.tag{2,i});
        
        % h1s(t) = ...
        text('Parent', my.objHandl(2), ...
            'Position', [bg.X_t2(i) bg.Y_t(i) 0], ...
            'Color', [0 0.3 0.8], ...
            'EraseMode', 'background', ...
            'FontAngle', 'italic', ...
            'FontSize',8.5, ...
            'String', my.tag{1,i}, ...
            'Tag', my.tag{1,i});
        % percents
        text('Parent', my.objHandl(2), ...
            'Position', [bg.X_t5(i) bg.Y_t3(i) 0], ...
            'Color', [0.48 0.15 0.6], ...
            'HorizontalAlignment','right', ...
            'EraseMode', 'background', ...
            'FontAngle', 'italic', ...
            'FontSize',8.5, ...
            'String', 0, ...
            'Tag', my.tag{5,i});
        % '%'
        text('Parent', my.objHandl(2), ...
            'Position', [bg.X_t5(i)+5 bg.Y_t3(i) 0], ...
            'Color', [0.48 0.15 0.6], ...
            'EraseMode', 'background', ...
            'FontAngle', 'italic', ...
            'FontSize',8.5, ...
            'String', '%');
    end
    
    for i = 1 : 3,
        % object which display the actual liquid flow
        handle_t2(i) = text('Parent', my.objHandl(2),...
            'EraseMode', 'background', ...
            'HorizontalAlignment','left', ...
            'FontAngle', 'italic', ...
            'FontSize', 8.5, ...
            'Position', [bg.X_t3(i) bg.Y_t2(i) 0], ...
            'String', '0.00', ...
            'Tag', my.tag{3,i});
        
        % q0s= q1(t)=
        text('Parent', my.objHandl(2), ...
            'Position', [bg.X_t4(i) bg.Y_t2(i) 0], ...
            'EraseMode', 'background', ...
            'FontAngle', 'italic', ...
            'FontSize',8.5, ...
            'String', my.tag{4,i}, ...
            'Tag', my.tag{4,i});
    end
    my.textHandl = [handle_t1 handle_t2];
    clear i handle_t1 handle_t2;
end
tic
% --------
function sys = mdlUpdate(u, kk, intr, zas)
global  my bg level_last data_last koef onetime tank_max

% draw the liquid
[level_last, onetime] = create_frame(u(1:2), u(3), intr, level_last, koef, onetime, kk, zas);
draw_background('continue', intr);

sys = [];
% --------

% -------
function mdlOutput
% make simulation slower
timeS = toc;
if (timeS < 0.5)
    pause(0.5-timeS);
end
tic
% Nothing happens here
% --------
function mdlTerminate
% Nothing happens here
% --------

% --------
function [backLL, onetime] = create_frame(level, q, intr, level_last, koef, onetime, kk, zas)
global bg my tank_max

color_liquid = [0.62 0.753 0.922];
color_liquid2 = [0.55 0.66 0.93];

tank_posit = [bg.bottom_part11(1,2) bg.bottom_part1(1,2)];    % y-axes of left down point of the tank
% determine the liquid height in the tanks
Y1 = tank_posit(1) + koef*level(1);
Y2 = tank_posit(2) + koef*level(2);
X = [bg.bottom_part11(1,1) bg.bottom_part21(11,1) bg.bottom_part1(1,1) bg.bottom_part2(11,1)];

% display the percent of fill of a tank
perLT = 100*(koef*level(1)/tank_max);
perRT = 100*(koef*level(2)/tank_max);
set(findobj('Tag',my.tag{5,1}),'String', sprintf('%.0f', perLT));
set(findobj('Tag',my.tag{5,2}),'String', sprintf('%.0f', perRT));

% interuption of the simulation in case of insufficient magnitude of a tank
if perRT  >= 100 | perLT >= 100
    modl = bdroot(find_system('MaskType','Zasobniky'));
    if ~isempty(modl)
        set_param(modl{1}, 'SimulationCommand', 'stop')
    end
    disp('>> simulacia bola predcasne ukoncena, nedostatocna velkost zasobnikov')
    disp('zmente hodnotu hzas a spustite simulaciu odznova')
end

% display the actual liquid flow if the value was changed
switch intr
    case 0
        q1 = kk(1)*sqrt(level(1));
    case 1,
        q1 = kk(1)*sqrt(level(1)-level(2));
end
q2 = kk(2)*sqrt(level(2));
set(findobj('Tag',my.tag{3,2}),'String', sprintf('%.2f', q1));
set(findobj('Tag',my.tag{3,3}),'String',sprintf('%.2f', q2));

if zas(2) ~= q
    set(findobj('Tag',my.tag{3,1}),'String', sprintf('%.2f', q))
    if onetime(6) == 0
        set(findobj('Tag',my.tag{4,1}),'String', 'q0(t) =')
        set(findobj('Tag',my.tag{1,1}),'String','h1(t) =')
        set(findobj('Tag',my.tag{1,2}),'String','h2(t) =')
        onetime(6) = onetime(6)+1;
    end
    percF0 = q/zas(3);
else
    if onetime(7) == 0
        set(findobj('Tag',my.tag{3,1}),'String', sprintf('%.2f', zas(2)))
        onetime(7) = onetime(7)+1;
    end
    percF0 = zas(2)/zas(3);
end
percF1 = q1/zas(3);
percF2 = q2/zas(3);

% draw liquid entering the left tank
if q ~= 0
    set(findobj('Tag', 'StreamL','Visible','off'),'Visible','on')

    if (percF0 > 0 & percF0 <= 0.3)
        X_stream = [bg.pipeline_left2(12,1)*ones(1,2)+2*ones(1,2) bg.pipeline_left2(12,1)*ones(1,2)+9*ones(1,2)];
    elseif (percF0 > 0.3 & percF0 <= 0.7)
        X_stream = [bg.pipeline_left2(12,1)*ones(1,2)+2*ones(1,2) bg.pipeline_left2(12,1)*ones(1,2)+16*ones(1,2)];
    else
        X_stream = [bg.pipeline_left2(12,1)*ones(1,2)+2*ones(1,2) bg.pipeline_left2(12,1)*ones(1,2)+23*ones(1,2)];
    end
    Y_stream = [Y1 bg.roof_part31(4,2) bg.roof_part31(4,2) Y1];
    if onetime(1) == 0
        fill(X_stream, Y_stream, color_liquid, 'EdgeColor', 'none','EraseMode','background', 'Tag', 'StreamL');
        onetime(1) = onetime(1)+1;
    else
        set(findobj('Tag', 'StreamL'), 'XData', X_stream, 'YData', Y_stream)
    end
else
    set(findobj('Tag', 'StreamL','Visible','on'),'Visible','off')
end
% draw liquid entering the right tank
if intr == 0
    if q1 ~= 0
        set(findobj('Tag', 'StreamM','Visible','off'),'Visible','on')
        
        if (percF1 > 0 & percF1 <= 0.3)
            X_stream = [bg.pipeline_m2(7,1)*ones(1,2)+2*ones(1,2) bg.pipeline_m2(7,1)*ones(1,2)+9*ones(1,2)];
        elseif (percF1 > 0.3 & percF1 <= 0.7)
            X_stream = [bg.pipeline_m2(7,1)*ones(1,2)+2*ones(1,2) bg.pipeline_m2(7,1)*ones(1,2)+16*ones(1,2)];
        else
            X_stream = [bg.pipeline_m2(7,1)*ones(1,2)+2*ones(1,2) bg.pipeline_m2(7,1)*ones(1,2)+23*ones(1,2)];
        end
        Y_stream = [Y2 bg.roof_part3(4,2) bg.roof_part3(4,2) Y2];
        if onetime(2) == 0
            fill(X_stream, Y_stream, color_liquid, 'EdgeColor', 'none','EraseMode','background', 'Tag', 'StreamM');
            onetime(2) = onetime(2)+1;
        else
            set(findobj('Tag', 'StreamM'), 'XData', X_stream, 'YData', Y_stream)
        end
    else
        set(findobj('Tag', 'StreamM','Visible','on'),'Visible','off')
    end
end

% draw liquid streaming from the right tank
if q2 ~= 0
    set(findobj('Tag', 'StreamR','Visible','off'),'Visible','on')
    
    if (percF2 > 0 & percF2 <= 0.3)
        X_stream = [bg.pipeline_right2(5,1)*ones(1,2)+2*ones(1,2) bg.pipeline_right2(5,1)*ones(1,2)+9*ones(1,2)];
    elseif (percF2 > 0.3 & percF2 <= 0.7)
        X_stream = [bg.pipeline_right2(5,1)*ones(1,2)+2*ones(1,2) bg.pipeline_right2(5,1)*ones(1,2)+16*ones(1,2)];
    else
        X_stream = [bg.pipeline_right2(5,1)*ones(1,2)+2*ones(1,2) bg.pipeline_right2(5,1)*ones(1,2)+23*ones(1,2)];
    end
    Y_stream = [bg.bottom_tank(3,2) bg.pipeline_right2(7,2) bg.pipeline_right2(7,2) bg.bottom_tank(3,2)];
    if onetime(3) == 0
        fill(X_stream, Y_stream, color_liquid, 'EdgeColor', 'none','EraseMode','background','Tag','StreamR');
        onetime(3) = onetime(3)+1;
    else
        set(findobj('Tag', 'StreamR'), 'XData', X_stream, 'YData', Y_stream)
    end
else
    set(findobj('Tag', 'StreamR','Visible','on'),'Visible','off')
end

% draw liquid in the left tank
if level_last(1) ~= level(1)
    Z = [0 koef*level(1)];
    if onetime(4) == 0        
        % draw the first part of the liquid in left tank
        fill([X(1) X(1) X(2) X(2) X(1)],[Y1 tank_posit(1) tank_posit(1) Y1 Y1], ...
            color_liquid, 'EdgeColor', 'none','Tag','wtank1');
        
        for i = 1 : 2
            % second part of the liquid
            h = fill([bg.bottom_part11(11,1) bg.bottom_part21(1,1) bg.bottom_part21(11,1) bg.bottom_part41(1,1)  bg.bottom_part31(11,1) bg.bottom_part31(1,1)], ...
                [Z(i)+bg.bottom_part11(11,2) Z(i)+bg.bottom_part21(1,2) Z(i)+bg.bottom_part21(11,2) ...
                    Z(i)+bg.bottom_part41(1,2)  Z(i)+bg.bottom_part31(11,2) Z(i)+bg.bottom_part31(1,2)], ...
                color_liquid2,'LineWidth',1.2,'Tag', 'wlev1');
            % TOMUTO NEROZUMIEM h ~= ob_wl2, ako je to mozne ????
            ob_wl2 = findobj('Tag', 'wlev1');
        end
        onetime(4) = onetime(4) + 1;
    else
        ob_w11 = findobj('Tag','wtank1');
        set(ob_w11, 'YData', [Y1 tank_posit(1) tank_posit(1) Y1 Y1])
        
        set(findobj('Tag', 'wlev1','Visible','off'),'Visible','on')
        
        ob_wl2 = findobj('Tag', 'wlev1');
            set(ob_wl2(1), 'YData', [Z(2)+bg.bottom_part11(11,2) Z(2)+bg.bottom_part21(1,2) Z(2)+bg.bottom_part21(11,2) ...
                    Z(2)+bg.bottom_part41(1,2)  Z(2)+bg.bottom_part31(11,2) Z(2)+bg.bottom_part31(1,2)])
        if level(1) == 0
            set(findobj('Tag', 'wlev1'),'Visible','off')
        end
    end
end

%draw liquid in the right tank
if level_last(2) ~= level(2)
    Z = [0 koef*level(2)];
    if onetime(5) == 0
        % draw the first part of liquid in the right tank
        fill([X(3) X(3) X(4) X(4) X(3)],[Y2 tank_posit(2) tank_posit(2) Y2 Y2], ...
            color_liquid, 'EdgeColor', 'none','Tag', 'wtank2');
        for i = 1 : 2
            % second part of the liquid
            fill([bg.bottom_part1(11,1) bg.bottom_part2(1,1) bg.bottom_part2(11,1) bg.bottom_part4(1,1)  bg.bottom_part3(11,1) bg.bottom_part3(1,1)], ...
                [Z(i)+bg.bottom_part1(11,2) Z(i)+bg.bottom_part2(1,2) Z(i)+bg.bottom_part2(11,2) ...
                    Z(i)+bg.bottom_part4(1,2)  Z(i)+bg.bottom_part3(11,2) Z(i)+bg.bottom_part3(1,2)], ...
                color_liquid2,'Tag', 'wlev2');
        end
        onetime(5) = onetime(5) + 1;
    else
        ob_w21 = findobj('Tag', 'wtank2');
        set(ob_w21, 'YData', [Y2 tank_posit(2) tank_posit(2) Y2 Y2])
        
        set(findobj('Tag', 'wlev2','Visible','off'),'Visible','on')
        ob_w22 = findobj('Tag', 'wlev2');
            set(ob_w22(1), 'YData', [Z(2)+bg.bottom_part1(11,2) Z(2)+bg.bottom_part2(1,2) Z(2)+bg.bottom_part2(11,2) ...
                    Z(2)+bg.bottom_part4(1,2)  Z(2)+bg.bottom_part3(11,2) Z(2)+bg.bottom_part3(1,2)])
        if level(2) == 0
            set(findobj('Tag', 'wlev2'),'Visible','off')
        end
    end
end

% display the actual liquid level if the value was changed
for i = 1:2
    if level_last(i) ~= level(i)
        set(findobj('Tag', my.tag{2,i}),'string',sprintf('%.2f',level(i)));
        level_last(i) = level(i);
    end
end
backLL = level_last;
% -------

% % -------
% function [x1,x2] = tankeqpoint(q, kk, intr)
% % the computation of stationary points
% 
% switch intr
%     case 0
%         % the stationary liquid levels of tanks without interaction
%         x = [(q/kk(1))^2 ((q/kk(1))^2*(kk(1)/kk(2))^2)];
%         
%     case 1
%         % with interaction
%         Z = [ 1 -1 ; 1 -((kk(2)/kk(1))^2+1)];
%         Y = [(q/kk(1))^2 ; 0];
%         x = (Z\Y)';
% end
% 
% x1 = x(1);
% x2 = x(2);
% % ---------

% ---------
function draw_background(phase, intr)
global bg
color_liquid = [0.62 0.753 0.922];
color_pipeline = [0.93 0.93 0.93];

% color of the middle pipeline
fill([bg.pipeline_m1(:,1); flipud(bg.pipeline_m2(:,1))], ...
    [bg.pipeline_m1(:,2); flipud(bg.pipeline_m2(:,2))], ...
    color_pipeline, 'EdgeColor', 'none', 'Tag', 'touch');
% color of the right pipeline
fill([bg.pipeline_right1(:,1); flipud(bg.pipeline_right2(:,1))], ...
    [bg.pipeline_right1(:,2); flipud(bg.pipeline_right2(:,2))], ...
    color_pipeline, 'EdgeColor', 'none', 'Tag', 'touch');
plot(bg.pipeline_m1(:,1) , bg.pipeline_m1(:,2), ...
    'Color', [0 0 0], 'Tag', 'touch');
plot(bg.left_side(:,1) , bg.left_side(:,2), bg.left_side1(:,1) , bg.left_side1(:,2), ...
    'Color', [0 0 0], 'Tag', 'touch');
plot(bg.pipeline_m2(:,1) , bg.pipeline_m2(:,2), ...
    'Color', [0 0 0], 'Tag', 'touch');
plot(bg.pipeline_right1(:,1) , bg.pipeline_right1(:,2), ...
    'Color', [0 0 0], 'Tag', 'touch');
plot(bg.pipeline_right2(:,1) , bg.pipeline_right2(:,2), ...
    'Color', [0 0 0], 'Tag', 'touch');
plot(bg.out(:,1) , bg.out(:,2), bg.out1(:,1) , bg.out1(:,2), ...
    'Color', [0 0 0], 'Tag', 'touch');

if strcmp(phase,'continue') == 0
    fill([bg.pipeline_left1(6:12,1); flipud(bg.pipeline_left2(6:12,1))], ...
        [bg.pipeline_left1(6:12,2); flipud(bg.pipeline_left2(6:12,2))], ...
        color_pipeline, 'EdgeColor', 'none');
    fill([bg.pump(2,1); bg.pipeline_left1(2:6,1); flipud(bg.pipeline_left2(1:6,1))], ...
        [bg.pump(2,2); bg.pipeline_left1(2:6,2); flipud(bg.pipeline_left2(2:6,2)); bg.pump(2,2)], ...
        color_pipeline, 'EdgeColor', 'none');
    fill([bg.pipeline_left1(1,1); bg.pipeline_left1(1,1); bg.pipeline_left2(1,1); bg.pipeline_left2(1,1)], ...
        [bg.bottom_tank(3,2); bg.pump(7,2); bg.pump(6,2); bg.bottom_tank(3,2)], ...
        color_pipeline, 'EdgeColor', 'none');
    plot(bg.right_side(:,1) , bg.right_side(:,2), bg.right_side1(:,1) , bg.right_side1(:,2), ...
        'Color', [0 0 0]);
    % liquid in the lower big tank
    fill(bg.bottom_tank([3,6,5,4,3],1), ...
        bg.bottom_tank([3,6,5,4,3],2), ...
        color_liquid, 'EdgeColor', 'none');
    plot(bg.pipeline_left1(:,1) , bg.pipeline_left1(:,2), ...
        'Color', [0 0 0]);
    plot(bg.pipeline_left2(:,1) , bg.pipeline_left2(:,2), ...
        'Color', [0 0 0]);
    % color of the lower big tank
    fill(bg.bottom_tank(:,1), ...
        bg.bottom_tank(:,2), ...
        color_pipeline, 'EdgeColor', 'none');
    % color of the pump
    fill(bg.pump(:,1), bg.pump(:,2), 0.1*[1 1 1]);
    plot(bg.pump(:,1) , bg.pump(:,2), ...
        'Color', [0 0 0]);
    plot(bg.bottom_tank(:,1), bg.bottom_tank(:,2), ...
        'Color', 0.4*[1 1 1]);
    plot(bg.bottom_part1(:,1) , bg.bottom_part1(:,2), bg.bottom_part11(:,1) , bg.bottom_part11(:,2), ...
        'Color', [0 0 0]);
    plot(bg.bottom_part2(:,1) , bg.bottom_part2(:,2), bg.bottom_part21(:,1) , bg.bottom_part21(:,2), ...
        'Color', [0 0 0]);
    plot(bg.bottom_part3(:,1) , bg.bottom_part3(:,2), bg.bottom_part31(:,1) , bg.bottom_part31(:,2), ...
        'Color', [0.5 0.5 0.5]);
    plot(bg.bottom_part4(:,1) , bg.bottom_part4(:,2), bg.bottom_part41(:,1) , bg.bottom_part41(:,2), ...
        'Color', [0.5 0.5 0.5]);
    plot(bg.down_1(:,1) , bg.down_1(:,2), bg.down_11(:,1) , bg.down_11(:,2), ...
        'Color', [0 0 0]);
    plot(bg.down_2(:,1) , bg.down_2(:,2), bg.down_21(:,1) , bg.down_21(:,2), ...
        'Color', [0.5 0.5 0.5]);
    plot(bg.up_1(:,1) , bg.up_1(:,2), bg.up_11(:,1) , bg.up_11(:,2), ...
        'Color', [0 0 0]);
    plot(bg.up_2(:,1) , bg.up_2(:,2), bg.up_21(:,1) , bg.up_21(:,2), ...
        'Color', [0.5 0.5 0.5]);
    plot(bg.top_part1(:,1) , bg.top_part1(:,2), bg.top_part11(:,1) , bg.top_part11(:,2), ...
        'Color', [0 0 0]);
    plot(bg.top_part2(:,1) , bg.top_part2(:,2), bg.top_part21(:,1) , bg.top_part21(:,2), ...
        'Color', [0 0 0]);
    plot(bg.top_part3(:,1) , bg.top_part3(:,2), bg.top_part31(:,1) , bg.top_part31(:,2), ...
        'Color', [0.5 0.5 0.5]);
    plot(bg.top_part4(:,1) , bg.top_part4(:,2), bg.top_part41(:,1) , bg.top_part41(:,2), ...
        'Color', [0.5 0.5 0.5]);
    plot(bg.roof_part1(:,1) , bg.roof_part1(:,2), bg.roof_part11(:,1) , bg.roof_part11(:,2), ...
        'Color', [0 0 0]);
    plot(bg.roof_part2(:,1) , bg.roof_part2(:,2), bg.roof_part21(:,1) , bg.roof_part21(:,2), ...
        'Color', [0 0 0]);
    plot(bg.roof_part3(:,1) , bg.roof_part3(:,2), bg.roof_part31(:,1) , bg.roof_part31(:,2), ...
        'Color', [0 0 0]);
end 
m = findobj('Tag', 'touch');
if size(m,1) > 10
    delete(m(11:size(m,1)))
end
% --------

% --------
function bg = background(intr)
global bg

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% IT'S POSSIBLE TO CHANGE
% the dimension of tanks approximately +50/-100
movement_x = 180;       % width
movement_y = 250;       % higth
% width of pipeline
p_width = 28;
% positions of tanks without and with interaction
M_x = [250 500 200 600];     % in the direct of x-axis
M_y = [700 -350 400 0];      % in the direct of y-axis
% editing of the higth of left pipeline
ky = 50;
% editing of the end of right pipeline / the y-axes of end point/
pr_end = 100;
% sometimes it is necessary to change konstankts (kmr,kl) in front of p_width
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

bottom_tank = [17.5 62.5 62.5 62.5 1437.5 1437.5 1437.5 1482.5 1477.5 1462.5 1462.5 37.5 37.5 22.5 17.5; ...
        225 217.5 180 25 25 180 217.5 225 187.5 180 0 0 180 187.5 225]';
bottom_part1 =[ 11 19 24 30 40 50 67 74 88 114 150; ...
        12 10 9 8 8 7 6 6 5 5 4]';
s_b = size(bottom_part1);
bottom_part2 = zeros(s_b);
bottom_part3 = zeros(s_b);
bottom_part4 = zeros(s_b);

roof_part1 =[ 11 30 45 62 78 90 94 104 115 122 130 140 152; ...
        (movement_y * ones(1,13)+ [12 20 24 28 30 32 32 33 33.5 34.5 35 36 36]) ]';
s_r = size(roof_part1);
roof_part2 = zeros(s_r);

k = s_b(1);
bottom_part2(1,1) = bottom_part1(s_b(1),1) + movement_x;
bottom_part2(k,2) = bottom_part1(1,2);
bottom_part3(1,:) = bottom_part1(1,:);

for i = 2 : s_b(1),
    bottom_part2(i,1) = bottom_part2(i-1)+(bottom_part1((s_b(1)-(i-2)),1)-bottom_part1((s_b(1)-(i-2)-1),1));
    bottom_part2((i-1),2) = bottom_part1(k,2);
    
    bottom_part3(i,2) = bottom_part3((i-1),2)+(bottom_part1((i-1),2)-bottom_part1(i,2));
    bottom_part3(i,1) = bottom_part1(i,1);
    
    k = k - 1;
end

k = s_b(1);
bottom_part4(k,:) = bottom_part2(k,:);
for i = 2 : s_b(1),
    bottom_part4((k-1),2) = bottom_part4(k,2)+(bottom_part2(k,2)-bottom_part2((k-1),2));
    bottom_part4((k-1),1) = bottom_part2((k-1),1);
    k = k - 1;
end

top_part1 = [( bottom_part1(:,1))'; ...
        (movement_y*ones(s_b(1),1)+ bottom_part1(:,2))']';
top_part2 = [( bottom_part2(:,1))'; ...
        (movement_y*ones(s_b(1),1)+ bottom_part2(:,2))']';
top_part3 = [( bottom_part3(:,1))'; ...
        (movement_y*ones(s_b(1),1)+ bottom_part3(:,2))']';
top_part4 = [( bottom_part4(:,1))'; ...
        (movement_y*ones(s_b(1),1)+ bottom_part4(:,2))']';
s_t = size(top_part1);

down_1 = [bottom_part1(s_b(1),1) bottom_part2(1,1); ...
        bottom_part1(s_b(1),2) bottom_part2(1,2)]';
down_2 = [bottom_part3(s_b(1),1) bottom_part4(1,1); ...
        bottom_part3(s_b(1),2) bottom_part4(1,2)]';
s_ud = size(down_1);

up_1 = [top_part1(s_b(1),1) top_part2(1,1); ...
        top_part1(s_b(1),2) top_part2(1,2)]';
up_2 = [top_part3(s_b(1),1) top_part4(1,1); ...
        top_part3(s_b(1),2) top_part4(1,2)]';

kx = 0.80*(movement_x-123);
ko = 0.15*ky;
out = [ kx+409 kx+407.6 kx+406.3 kx+406.3 kx+406.3 kx+407.5 kx+407.8 kx+409.2 kx+410.6; ...
        ko+12.8 ko+15.2 ko+18.8 ko+24.7 ko+28.3 ko+30.9 ko+33.2 ko+35.5 ko+36.6]';
s_i = size(out);

left_side = [bottom_part1(1,1) top_part1(1,1); ...
        bottom_part1(1,2) top_part1(1,2)]';
right_side = [bottom_part2(11,1) top_part2(11,1); ...
        out(s_i(1),2) top_part2(11,2)]';

s_lr = size(left_side);

roof_part2(1,:) = right_side(2,:);
for i = 2 : s_r(1),
    roof_part2(i,1) = roof_part2((i-1),1)+(roof_part1((i-1),1)-roof_part1(i,1));
    roof_part2(i,2) = roof_part1(i,2);
end
s_r12 = size(roof_part1);

semidiameter_inflow = (roof_part1(s_r(1),1)+roof_part2(s_r(1),1))/2;
roof_part3 = [ roof_part1(s_r(1),1) semidiameter_inflow-(p_width/2) semidiameter_inflow-(p_width/2) semidiameter_inflow+15 semidiameter_inflow+15 roof_part2(s_r(1),1); ...
        roof_part1(s_r(1),2) roof_part1(s_r(1),2) roof_part1(s_r(1),2)+13 roof_part2(s_r(1),2)+13 roof_part2(s_r(1),2) roof_part2(s_r(1),2)]';
s_r3 = size(roof_part3);

switch intr
    case 0
        % without interaction
        move_x =[M_x(1) M_x(2)];
        move_y =[M_y(1) M_y(2)];
    case 1
        % with interaction
        move_x =[M_x(3) M_x(4)];
        move_y =[M_y(3) M_y(4)];
end

i = 1;
for i = 1 : 2,
    bottom_part1 = bottom_part1 + [move_x(i)*ones(s_b(1),1) move_y(i)*ones(s_b(1),1)];
    bottom_part2 = bottom_part2 + [move_x(i)*ones(s_b(1),1) move_y(i)*ones(s_b(1),1)];
    bottom_part3 = bottom_part3 + [move_x(i)*ones(s_b(1),1) move_y(i)*ones(s_b(1),1)];
    bottom_part4 = bottom_part4 + [move_x(i)*ones(s_b(1),1) move_y(i)*ones(s_b(1),1)];
    top_part1 = top_part1 + [move_x(i)*ones(s_b(1),1) move_y(i)*ones(s_b(1),1)];
    top_part2 = top_part2 + [move_x(i)*ones(s_b(1),1) move_y(i)*ones(s_b(1),1)];
    top_part3 = top_part3 + [move_x(i)*ones(s_b(1),1) move_y(i)*ones(s_b(1),1)];
    top_part4 = top_part4 + [move_x(i)*ones(s_b(1),1) move_y(i)*ones(s_b(1),1)];
    down_1 = down_1 + [move_x(i)*ones(s_ud(1),1) move_y(i)*ones(s_ud(1),1)];
    down_2 = down_2 + [move_x(i)*ones(s_ud(1),1) move_y(i)*ones(s_ud(1),1)];
    up_1 = up_1 + [move_x(i)*ones(s_ud(1),1) move_y(i)*ones(s_ud(1),1)];
    up_2 = up_2 + [move_x(i)*ones(s_ud(1),1) move_y(i)*ones(s_ud(1),1)];
    left_side = left_side + [move_x(i)*ones(s_lr(1),1) move_y(i)*ones(s_lr(1),1)];
    right_side = right_side + [move_x(i)*ones(s_lr(1),1) move_y(i)*ones(s_lr(1),1)];
    roof_part1 = roof_part1 + [move_x(i)*ones(s_r12(1),1) move_y(i)*ones(s_r12(1),1)];
    roof_part2 = roof_part2 + [move_x(i)*ones(s_r12(1),1) move_y(i)*ones(s_r12(1),1)];
    roof_part3 = roof_part3 + [move_x(i)*ones(s_r3(1),1) move_y(i)*ones(s_r3(1),1)];
    out = out + [move_x(i)*ones(s_i(1),1) move_y(i)*ones(s_i(1),1)];
    
    if i == 1
        bottom_part11 = bottom_part1;
        bottom_part21 = bottom_part2;
        bottom_part31 = bottom_part3;
        bottom_part41 = bottom_part4;
        top_part11 = top_part1;
        top_part21 = top_part2;
        top_part31 = top_part3;
        top_part41 = top_part4;
        down_11 = down_1;
        down_21 = down_2;
        up_11 = up_1;
        up_21 = up_2;
        left_side1 = left_side;
        right_side1 = right_side;
        roof_part11 = roof_part1;
        roof_part21 = roof_part2;
        roof_part31 = roof_part3;
        out1 = out;
    end
end

if intr == 1
    roof_part3 = zeros(2,2);
    roof_part3 = [ roof_part1(s_r(1),1) roof_part2(s_r(1),1); ...
            roof_part1(s_r(1),2) roof_part2(s_r(1),2)]';
end

kx = 0.5*(movement_x-123);
px = [move_x(1)-200 move_x(2)-400];
% sometimes it is necessary to change konstankts in front of p_width
kl = 0.7;   % initial value = 1
kmr = 1;  % initial value = 1

if intr == 1
    ky = ky - 300;
end
pipeline_left1 = [ 98 98.5 100 101.5  103 103.5 px(1)+kx+418  px(1)+kx+423.5  px(1)+kx+424  px(1)+kx+425 px(1)+kx+425 roof_part31(4,1); ...
        65 ky+1000 ky+1009.7 ky+1019.1 ky+1024   ky+1025  ky+1025 ky+1018.5 ky+1011.7 ky+1007 ky+1005 roof_part31(4,2)]';

pipeline_left2 = [ (pipeline_left1(1:6,1)+ p_width*ones(6,1))' (pipeline_left1(7:11,1)- p_width*ones(5,1))' roof_part31(3,1); ...
        pipeline_left1(1,2) (pipeline_left1(2:11,2)- kl*p_width*ones(10,1))' roof_part31(3,2)]';

kk = [720 640 725 585];

switch intr
    case 0
        pipeline_m1 = [out1(s_i(1),1)  roof_part3(3,1) px(2)+(pipeline_left1(7:11,1)+ 403*ones(5,1))' roof_part3(4,1); ...
                out1(s_i(1),2) out1(s_i(1),2) ko+(pipeline_left1(7:11,2)- 290*ones(5,1))'-ky roof_part3(4,2)]';
        pipeline_m2 = [out1(s_i(1),1) (pipeline_m1(3:7,1) - p_width*ones(5,1))' roof_part3(3,1); ...
                out1(1,2) (pipeline_m1(3:7,2)- kmr*p_width*ones(5,1))' roof_part3(3,2)]';
        p_r1 = [out(s_i(1),1) px(2)+(pipeline_left1(7:11,1)+kk(1)*ones(5,1))'; ...
                out(s_i(1),2) (ko+(pipeline_left1(7:11,2)-kk(2)*ones(5,1))'-ky)];
    case 1
        pipeline_m1 = [out1(s_i(1),1) left_side(1,1); ...
                out1(s_i(1),2) out1(s_i(1),2)]';
        pipeline_m2 = [pipeline_m1(:,1)'; ...
                (pipeline_m1(:,2)-kmr*p_width*ones(2,1))']';
        p_r1 = [out(s_i(1),1) px(2)+(pipeline_left1(7:11,1)+kk(3)*ones(5,1))'; ...
                out(s_i(1),2) (ko+(pipeline_left1(7:11,2)-kk(4)*ones(5,1))'-ky)];
end
s_pm = size(pipeline_m1);

pipeline_right1 = [p_r1(1,:) p_r1(1,6); ...
        p_r1(2,:) p_r1(2,6)-pr_end]';
pipeline_right2 = [out(s_i(1),1) (pipeline_right1(2:7,1) - kmr*p_width*ones(6,1))'; ...
        out(1,2) (pipeline_right1(2:6,2)- kmr*p_width*ones(5,1))' pipeline_right1(7,2)]';

pump = [0 24 48 72 72 48 24 0 0; ...
        174 180 180 174 6 0 0 6 174]';
pump = [(pump(:,1) + 77*ones(9,1)) (pump(:,2) + 300*ones(9,1))];

% possition of a text which display the curent value of a flow and a liquid hight
if intr == 0
    X_t1 = [445 935];
    X_t2 = [290 780];
    X_t3 = [705 1205 1410];
    X_t4 = [535 1035 1235];
    X_t5 = [815 1320];
    
    Y_t = [670 320];
    Y_t2 = [1045 695 430];
    Y_t3 = [920 570];
    
elseif intr == 1
    X_t1 = [410 1000];
    Y_t = [360 360];
    X_t2 = [260 850];
    
    X_t3 = [305 790 1195];
    X_t4 = [130 610 1015];
    X_t5 = [570 1370];
    Y_t2 = [825 725 265];
    Y_t3 = [770 640];
    
end

bg = struct(...
    'bottom_tank', bottom_tank, ...
    'bottom_part1',bottom_part1, 'bottom_part11',bottom_part11, ...
    'bottom_part2',bottom_part2, 'bottom_part21',bottom_part21, ...
    'bottom_part4',bottom_part4, 'bottom_part41',bottom_part41, ...
    'top_part1',top_part1, 'top_part11',top_part11, ...
    'top_part2',top_part2, 'top_part21',top_part21, ...
    'top_part3',top_part3, 'top_part31',top_part31, ...
    'top_part4',top_part4, 'top_part41',top_part41, ...
    'left_side',left_side, 'left_side1',left_side1, ...
    'right_side',right_side, 'right_side1',right_side1, ...
    'bottom_part3',bottom_part3, 'bottom_part31',bottom_part31, ...
    'down_1', down_1, 'down_11', down_11, ...
    'down_2', down_2, 'down_21', down_21, ...
    'up_1', up_1, 'up_11', up_11, ...
    'up_2', up_2, 'up_21', up_21, ...
    'roof_part1',roof_part1, 'roof_part11',roof_part11, ...
    'roof_part2',roof_part2, 'roof_part21',roof_part21, ...
    'roof_part3',roof_part3, 'roof_part31',roof_part31, ...
    'out',out, 'out1',out1, ...
    'pipeline_left1',pipeline_left1, ...
    'pipeline_left2',pipeline_left2, ...
    'pipeline_m1',pipeline_m1, ...
    'pipeline_m2',pipeline_m2, ...
    'pipeline_right1',pipeline_right1, ...
    'pipeline_right2',pipeline_right2, ...
    'pump',pump, ...
    'X_t1', X_t1, ...
    'X_t2', X_t2, ...
    'X_t3', X_t3, ...
    'X_t4', X_t4, ...
    'X_t5', X_t5, ...
    'Y_t', Y_t, ...
    'Y_t2', Y_t2, ...
    'Y_t3', Y_t3);
% --------